@@ -22,6 +22,7 @@ ACTIONS:
  search         search twitter (Beware: octothorpe, escape it)
  set            set your twitter status
  shell          login to the twitter shell
+ rate           get your current rate limit status (remaining API reqs)
 
 
 OPTIONS:
@@ -61,6 +62,14 @@ prompt: <twitter_shell_prompt e.g. '[cyan]twitter[R]> '>
  home directory.
 """
 
+from __future__ import print_function
+
+try:
+    input = __builtins__['raw_input']
+except AttributeError:
+    pass
+
+
 CONSUMER_KEY='uS6hO2sV6tDKIOeVjhnFnQ'
 CONSUMER_SECRET='MEYTOS97VvlHX7K1rwHPEqVpTSqZ71HtvoK4sVuYk'
 
@@ -70,16 +79,25 @@ from getopt import gnu_getopt as getopt, GetoptError
 from getpass import getpass
 import re
 import os.path
-from ConfigParser import SafeConfigParser
+import locale
+import string
+
+try:
+    from ConfigParser import SafeConfigParser
+except ImportError:
+    from configparser import ConfigParser as SafeConfigParser
 import datetime
-from urllib import quote
+try:
+    from urllib.parse import quote
+except ImportError:
+    from urllib2 import quote
 import webbrowser
 
-from api import Twitter, TwitterError
-from oauth import OAuth, write_token_file, read_token_file
-from oauth_dance import oauth_dance
-import ansi
-from util import smrt_input
+from .api import Twitter, TwitterError
+from .oauth import OAuth, write_token_file, read_token_file
+from .oauth_dance import oauth_dance
+from . import ansi
+from .util import smrt_input, printNicely
 
 OPTIONS = {
     'action': 'friends',
@@ -102,6 +120,8 @@ def parse_args(args, options):
                  'datestamp', 'no-ssl']
     short_opts = "e:p:f:h?rR:c:l:td"
     opts, extra_args = getopt(args, short_opts, long_opts)
+    extra_args = [arg.decode(locale.getpreferredencoding())
+                  for arg in extra_args]
 
     for opt, arg in opts:
         if opt in ('-f', '--format'):
@@ -149,7 +169,7 @@ def get_time_string(status, options, format="%a %b %d %H:%M:%S +0000 %Y"):
 
 class StatusFormatter(object):
     def __call__(self, status, options):
-        return (u"%s%s %s" %(
+        return ("%s%s %s" %(
             get_time_string(status, options),
             status['user']['screen_name'], status['text']))
 
@@ -159,14 +179,14 @@ class AnsiStatusFormatter(object):
 
     def __call__(self, status, options):
         colour = self._colourMap.colourFor(status['user']['screen_name'])
-        return (u"%s%s%s%s %s" %(
+        return ("%s%s%s%s %s" %(
             get_time_string(status, options),
             ansi.cmdColour(colour), status['user']['screen_name'],
             ansi.cmdReset(), status['text']))
 
 class VerboseStatusFormatter(object):
     def __call__(self, status, options):
-        return (u"-- %s (%s) on %s\n%s\n" %(
+        return ("-- %s (%s) on %s\n%s\n" %(
             status['user']['screen_name'],
             status['user']['location'],
             status['created_at'],
@@ -176,20 +196,20 @@ class URLStatusFormatter(object):
     urlmatch = re.compile(r'https?://\S+')
     def __call__(self, status, options):
         urls = self.urlmatch.findall(status['text'])
-        return u'\n'.join(urls) if urls else ""
+        return '\n'.join(urls) if urls else ""
 
 
 class ListsFormatter(object):
     def __call__(self, list):
         if list['description']:
-            list_str = u"%-30s (%s)" % (list['name'], list['description'])
+            list_str = "%-30s (%s)" % (list['name'], list['description'])
         else:
-            list_str = u"%-30s" % (list['name'])
-        return u"%s\n" % list_str
+            list_str = "%-30s" % (list['name'])
+        return "%s\n" % list_str
 
 class ListsVerboseFormatter(object):
     def __call__(self, list):
-        list_str = u"%-30s\n description: %s\n members: %s\n mode:%s\n" % (list['name'], list['description'], list['member_count'], list['mode'])
+        list_str = "%-30s\n description: %s\n members: %s\n mode:%s\n" % (list['name'], list['description'], list['member_count'], list['mode'])
         return list_str
 
 class AnsiListsFormatter(object):
@@ -198,22 +218,22 @@ class AnsiListsFormatter(object):
 
     def __call__(self, list):
         colour = self._colourMap.colourFor(list['name'])
-        return (u"%s%-15s%s %s" %(
+        return ("%s%-15s%s %s" %(
             ansi.cmdColour(colour), list['name'],
             ansi.cmdReset(), list['description']))
 
 
 class AdminFormatter(object):
     def __call__(self, action, user):
-        user_str = u"%s (%s)" %(user['screen_name'], user['name'])
+        user_str = "%s (%s)" %(user['screen_name'], user['name'])
         if action == "follow":
-            return u"You are now following %s.\n" %(user_str)
+            return "You are now following %s.\n" %(user_str)
         else:
-            return u"You are no longer following %s.\n" %(user_str)
+            return "You are no longer following %s.\n" %(user_str)
 
 class VerboseAdminFormatter(object):
     def __call__(self, action, user):
-        return(u"-- %s: %s (%s): %s" % (
+        return("-- %s: %s (%s): %s" % (
             "Following" if action == "follow" else "Leaving",
             user['screen_name'],
             user['name'],
@@ -221,7 +241,7 @@ class VerboseAdminFormatter(object):
 
 class SearchFormatter(object):
     def __call__(self, result, options):
-        return(u"%s%s %s" %(
+        return("%s%s %s" %(
             get_time_string(result, options, "%a, %d %b %Y %H:%M:%S +0000"),
             result['from_user'], result['text']))
 
@@ -232,7 +252,7 @@ class URLSearchFormatter(object):
     urlmatch = re.compile(r'https?://\S+')
     def __call__(self, result, options):
         urls = self.urlmatch.findall(result['text'])
-        return u'\n'.join(urls) if urls else ""
+        return '\n'.join(urls) if urls else ""
 
 class AnsiSearchFormatter(object):
     def __init__(self):
@@ -240,7 +260,7 @@ class AnsiSearchFormatter(object):
 
     def __call__(self, result, options):
         colour = self._colourMap.colourFor(result['from_user'])
-        return (u"%s%s%s%s %s" %(
+        return ("%s%s%s%s %s" %(
             get_time_string(result, options, "%a, %d %b %Y %H:%M:%S +0000"),
             ansi.cmdColour(colour), result['from_user'],
             ansi.cmdReset(), result['text']))
@@ -315,13 +335,13 @@ class Action(object):
 
         prompt = 'You really want to %s %s? ' %(subject, sample)
         try:
-            answer = raw_input(prompt).lower()
+            answer = input(prompt).lower()
             if careful:
                 return answer in ('yes', 'y')
             else:
                 return answer not in ('no', 'n')
         except EOFError:
-            print >>sys.stderr # Put Newline since Enter was never pressed
+            print(file=sys.stderr) # Put Newline since Enter was never pressed
             # TODO:
                 #   Figure out why on OS X the raw_input keeps raising
                 #   EOFError and is never able to reset and get more input
@@ -338,11 +358,12 @@ class Action(object):
             if (options['refresh'] and isinstance(action, StatusAction)):
                 while True:
                     doAction()
+                    sys.stdout.flush()
                     time.sleep(options['refresh_rate'])
             else:
                 doAction()
         except KeyboardInterrupt:
-            print >>sys.stderr, '\n[Keyboard Interrupt]'
+            print('\n[Keyboard Interrupt]', file=sys.stderr)
             pass
 
 class NoSuchActionError(Exception):
@@ -352,12 +373,6 @@ class NoSuchAction(Action):
     def __call__(self, twitter, options):
         raise NoSuchActionError("No such action: %s" %(options['action']))
 
-def printNicely(string):
-    if sys.stdout.encoding:
-        print string.encode(sys.stdout.encoding, 'replace')
-    else:
-        print string.encode('utf-8')
-
 class StatusAction(Action):
     def __call__(self, twitter, options):
         statuses = self.getStatuses(twitter, options)
@@ -376,7 +391,7 @@ class SearchAction(Action):
         # We need to bypass the TwitterCall parameter encoding, so we
         # don't encode the plus sign, so we have to encode it ourselves
         query_string = "+".join(
-            [quote(term.decode(get_term_encoding()))
+            [quote(term)
              for term in options['extra_args']])
 
         results = twitter.search(q=query_string)['results']
@@ -393,14 +408,14 @@ class AdminAction(Action):
         af = get_formatter('admin', options)
         try:
             user = self.getUser(twitter, options['extra_args'][0])
-        except TwitterError, e:
-            print "There was a problem following or leaving the specified user."
-            print "You may be trying to follow a user you are already following;"
-            print "Leaving a user you are not currently following;"
-            print "Or the user may not exist."
-            print "Sorry."
-            print
-            print e
+        except TwitterError as e:
+            print("There was a problem following or leaving the specified user.")
+            print("You may be trying to follow a user you are already following;")
+            print("Leaving a user you are not currently following;")
+            print("Or the user may not exist.")
+            print("Sorry.")
+            print()
+            print(e)
         else:
             printNicely(af(options['action'], user))
 
@@ -453,11 +468,36 @@ class LeaveAction(AdminAction):
 
 class SetStatusAction(Action):
     def __call__(self, twitter, options):
-        statusTxt = (" ".join(options['extra_args']).decode(get_term_encoding())
+        statusTxt = (" ".join(options['extra_args'])
                      if options['extra_args']
-                     else unicode(raw_input("message: ")))
-        status = (statusTxt.encode('utf8', 'replace'))
-        twitter.statuses.update(status=status)
+                     else str(input("message: ")))
+        replies = []
+        ptr = re.compile("@[\w_]+")
+        while statusTxt:
+            s = ptr.match(statusTxt)
+            if s and s.start() == 0:
+                replies.append(statusTxt[s.start():s.end()])
+                statusTxt = statusTxt[s.end()+1:]
+            else:
+                break
+        replies = " ".join(replies)
+        if len(replies) >= 140:
+            # just go back
+            statusTxt = replies
+            replies = ""
+
+        splitted = []
+        while statusTxt:
+            limit = 140 - len(replies)
+            if len(statusTxt) > limit:
+                end = string.rfind(statusTxt, ' ', 0, limit)
+            else:
+                end = limit
+            splitted.append(" ".join((replies,statusTxt[:end])))
+            statusTxt = statusTxt[end:]
+
+        for status in splitted:
+            twitter.statuses.update(status=status)
 
 class TwitterShell(Action):
 
@@ -476,33 +516,33 @@ class TwitterShell(Action):
         while True:
             options['action'] = ""
             try:
-                args = raw_input(prompt).split()
+                args = input(prompt).split()
                 parse_args(args, options)
                 if not options['action']:
                     continue
                 elif options['action'] == 'exit':
                     raise SystemExit(0)
                 elif options['action'] == 'shell':
-                    print >>sys.stderr, 'Sorry Xzibit does not work here!'
+                    print('Sorry Xzibit does not work here!', file=sys.stderr)
                     continue
                 elif options['action'] == 'help':
-                    print >>sys.stderr, '''\ntwitter> `action`\n
+                    print('''\ntwitter> `action`\n
                           The Shell Accepts all the command line actions along with:
 
                           exit    Leave the twitter shell (^D may also be used)
 
-                          Full CMD Line help is appended below for your convinience.'''
+                          Full CMD Line help is appended below for your convinience.''', file=sys.stderr)
                 Action()(twitter, options)
                 options['action'] = ''
-            except NoSuchActionError, e:
-                print >>sys.stderr, e
+            except NoSuchActionError as e:
+                print(e, file=sys.stderr)
             except KeyboardInterrupt:
-                print >>sys.stderr, '\n[Keyboard Interrupt]'
+                print('\n[Keyboard Interrupt]', file=sys.stderr)
             except EOFError:
-                print >>sys.stderr
+                print(file=sys.stderr)
                 leaving = self.ask(subject='Leave')
                 if not leaving:
-                    print >>sys.stderr, 'Excellent!'
+                    print('Excellent!', file=sys.stderr)
                 else:
                     raise SystemExit(0)
 
@@ -516,12 +556,19 @@ class PythonPromptAction(Action):
 
 class HelpAction(Action):
     def __call__(self, twitter, options):
-        print __doc__
+        print(__doc__)
 
 class DoNothingAction(Action):
     def __call__(self, twitter, options):
         pass
 
+class RateLimitStatus(Action):
+    def __call__(self, twitter, options):
+        rate = twitter.account.rate_limit_status()
+        print("Remaining API requests: %s / %s (hourly limit)" % (rate['remaining_hits'], rate['hourly_limit']))
+        print("Next reset in %ss (%s)" % (int(rate['reset_time_in_seconds']-time.time()),
+                                          time.asctime(time.localtime(rate['reset_time_in_seconds']))))
+
 actions = {
     'authorize' : DoNothingAction,
     'follow'    : FollowAction,
@@ -536,6 +583,7 @@ actions = {
     'search'    : SearchAction,
     'set'       : SetStatusAction,
     'shell'     : TwitterShell,
+    'rate'      : RateLimitStatus,
 }
 
 def loadConfig(filename):
@@ -552,9 +600,9 @@ def main(args=sys.argv[1:]):
     arg_options = {}
     try:
         parse_args(args, arg_options)
-    except GetoptError, e:
-        print >> sys.stderr, "I can't do that, %s." %(e)
-        print >> sys.stderr
+    except GetoptError as e:
+        print("I can't do that, %s." %(e), file=sys.stderr)
+        print(file=sys.stderr)
         raise SystemExit(1)
 
     config_path = os.path.expanduser(
@@ -566,13 +614,13 @@ def main(args=sys.argv[1:]):
     # arguments.
     options = dict(OPTIONS)
     for d in config_options, arg_options:
-        for k,v in d.items():
+        for k,v in list(d.items()):
             if v: options[k] = v
 
     if options['refresh'] and options['action'] not in (
         'friends', 'public', 'replies'):
-        print >> sys.stderr, "You can only refresh the friends, public, or replies actions."
-        print >> sys.stderr, "Use 'twitter -h' for help."
+        print("You can only refresh the friends, public, or replies actions.", file=sys.stderr)
+        print("Use 'twitter -h' for help.", file=sys.stderr)
         return 1
 
     oauth_filename = os.path.expanduser(options['oauth_filename'])
@@ -594,10 +642,10 @@ def main(args=sys.argv[1:]):
 
     try:
         Action()(twitter, options)
-    except NoSuchActionError, e:
-        print >>sys.stderr, e
+    except NoSuchActionError as e:
+        print(e, file=sys.stderr)
         raise SystemExit(1)
-    except TwitterError, e:
-        print >> sys.stderr, str(e)
-        print >> sys.stderr, "Use 'twitter -h' for help."
+    except TwitterError as e:
+        print(str(e), file=sys.stderr)
+        print("Use 'twitter -h' for help.", file=sys.stderr)
         raise SystemExit(1)
